==> beagle-2.1.5.tar.gz: beagle-2.1.5/beagle/GA/src/MutationESVecOp.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/MutationESVecOp.cpp
 *  \brief  Source code of class GA::MutationESVecOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.7 $
 *  $Date: 2004/04/25 23:12:19 $
 */

#include "beagle/GA.hpp"

#include <cfloat>
#include <cmath>
#include <algorithm>
#include <string>

using namespace Beagle;


/*!
 *  \brief Construct a evolution strategy mutation operator.
 *  \param inMutationPbName ES mutation probability parameter name used in register.
 *  \param inMinStrategyName Minimum strategy parameter parameter name.
 *  \param inName Name of the operator.
 */
GA::MutationESVecOp::MutationESVecOp(std::string inMutationPbName,
                                     std::string inMinStrategyName,
                                     std::string inName) :
  MutationOp(inMutationPbName, inName),
  mMinStrategyName(inMinStrategyName)
{ }


/*!
 *  \brief Initialize the ES mutation operator.
 *  \param ioSystem System of the evolution.
 */
void GA::MutationESVecOp::initialize(System& ioSystem)
{
  MutationOp::initialize(ioSystem);
  if(ioSystem.getRegister().isRegistered(mMutationPbName)) {
    ioSystem.getRegister().deleteEntry(mMutationPbName);
  }
  
  if(ioSystem.getRegister().isRegistered(mMutationPbName)) {
    mMutationProba = castHandleT<Float>(ioSystem.getRegister()[mMutationPbName]);
  } else {
    mMutationProba = new Float(float(1.0));
    Register::Description lDescription(
      "Individual ES mutation prob.",
      "Float",
      "1.0",
      "ES mutation probability for each individual."
    );
    ioSystem.getRegister().addEntry(mMutationPbName, mMutationProba, lDescription);
  }

  if(ioSystem.getRegister().isRegistered(mMinStrategyName)) {
    mMinStrategy = castHandleT<Float>(ioSystem.getRegister()[mMinStrategyName]);
  } else {
    mMinStrategy = new Float(float(0.01));
    Register::Description lDescription(
      "Minimum strategy parameter",
      "Float",
      "0.01",
      "Minimum mutation strategy parameter allowed."
    );
    ioSystem.getRegister().addEntry(mMinStrategyName, mMinStrategy, lDescription);
  }
}


/*!
 *  \brief Mutate an ES individual.
 *  \param ioIndividual ES individual to mutate.
 *  \param ioContext Context of the evolution.
 *  \return True as individual are always mutated.
 */
bool GA::MutationESVecOp::mutate(Beagle::Individual& ioIndividual, Context& ioContext)
{
  Beagle_ValidateParameterM(mMinStrategy->getWrappedValue()>=0.0,mMinStrategyName,"<0");

  Beagle_LogVerboseM(
    ioContext.getSystem().getLogger(),
    "mutation", "Beagle::GA::MutationESVecOp",
    std::string("Applying evolution strategy mutation to an individual")
  );

  for(unsigned int i=0; i<ioIndividual.size(); i++) {
    GA::ESVector::Handle lVector = castHandleT<GA::ESVector>(ioIndividual[i]);
    Beagle_LogVerboseM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("Mutating the ")+uint2ordinal(i+1)+" ES vector"
    );
    Beagle_LogDebugM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("The ES vector before mutation: ")+lVector->serialize()
    );

    float lT      = 1.0 / std::sqrt(2.0 * std::sqrt(double(lVector->size())));
    float lTPrime = 1.0 / std::sqrt(2.0 * lVector->size());
    float lN      = ioContext.getSystem().getRandomizer().rollGaussian(0.0, 1.0);
    for(unsigned int j=0; j<lVector->size(); j++) {
      float lNi = ioContext.getSystem().getRandomizer().rollGaussian(0.0, 1.0);
        
      (*lVector)[j].mStrategy *= std::exp((lTPrime * lN) + (lT * lNi));
      if((*lVector)[j].mStrategy < mMinStrategy->getWrappedValue())
        (*lVector)[j].mStrategy = mMinStrategy->getWrappedValue();
      (*lVector)[j].mValue += (*lVector)[j].mStrategy * lNi;

      Beagle_LogDebugM(
        ioContext.getSystem().getLogger(),
        "mutation", "Beagle::GA::MutationESVecOp",
        std::string("ES mutating by adding ")+dbl2str((*lVector)[j].mStrategy * lNi)+
        std::string(" to the value and multiplying the strategy by ")+
        dbl2str(std::exp((lTPrime * lN) + (lT * lNi)))+
        std::string(" to mutate the pair ")+uint2str(j)+
        std::string(" of the ES vector")
      );
    }
    
    Beagle_LogDebugM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("ES vector after mutation: ")+lVector->serialize()
    );
  }
  return true;
}

==> beagle-2.2.0.tar.gz: beagle-2.2.0/beagle/GA/src/MutationESVecOp.cpp <==
/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GA/src/MutationESVecOp.cpp
 *  \brief  Source code of class GA::MutationESVecOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.8 $
 *  $Date: 2004/10/26 16:53:59 $
 */

#include "beagle/GA.hpp"

#include <cfloat>
#include <cmath>
#include <algorithm>
#include <string>

using namespace Beagle;


/*!
 *  \brief Construct a evolution strategy mutation operator.
 *  \param inMutationPbName ES mutation probability parameter name used in register.
 *  \param inMinStrategyName Minimum strategy parameter parameter name.
 *  \param inName Name of the operator.
 */
GA::MutationESVecOp::MutationESVecOp(std::string inMutationPbName,
                                     std::string inMinStrategyName,
                                     std::string inName) :
  MutationOp(inMutationPbName, inName),
  mMinStrategyName(inMinStrategyName)
{ }


/*!
 *  \brief Initialize the ES mutation operator.
 *  \param ioSystem System of the evolution.
 */
void GA::MutationESVecOp::initialize(System& ioSystem)
{
  MutationOp::initialize(ioSystem);
  if(ioSystem.getRegister().isRegistered(mMutationPbName)) {
    ioSystem.getRegister().deleteEntry(mMutationPbName);
  }
  
  if(ioSystem.getRegister().isRegistered(mMutationPbName)) {
    mMutationProba = castHandleT<Float>(ioSystem.getRegister()[mMutationPbName]);
  } else {
    mMutationProba = new Float(1.0f);
    Register::Description lDescription(
      "Individual ES mutation prob.",
      "Float",
      "1.0",
      "ES mutation probability for each individual."
    );
    ioSystem.getRegister().addEntry(mMutationPbName, mMutationProba, lDescription);
  }

  if(ioSystem.getRegister().isRegistered(mMinStrategyName)) {
    mMinStrategy = castHandleT<Double>(ioSystem.getRegister()[mMinStrategyName]);
  } else {
    mMinStrategy = new Double(0.01);
    Register::Description lDescription(
      "Minimum strategy parameter",
      "Double",
      "0.01",
      "Minimum mutation strategy parameter allowed."
    );
    ioSystem.getRegister().addEntry(mMinStrategyName, mMinStrategy, lDescription);
  }

  if(ioSystem.getRegister().isRegistered("es.value.max")) {
    mMaxValue = castHandleT<Vector>(ioSystem.getRegister()["es.value.max"]);
  } else {
    mMaxValue = new Vector(1,DBL_MAX);
    std::ostringstream lOSS;
    lOSS << "ES vector maximum values. This can be a scalar, which limit the values for all ES, ";
    lOSS << "or a vector which limit the values individually. If the maximum value vector size is ";
    lOSS << "smaller than the ES vector size, the limit used for the last values of the ES vector ";
    lOSS << "is equal to the last value of the maximum value vector.";
    Register::Description lDescription(
      "Maximum ES vector values",
      "Vector",
      dbl2str(DBL_MAX),
      lOSS.str()
    );
    ioSystem.getRegister().addEntry("es.value.max", mMaxValue, lDescription);
  }

  if(ioSystem.getRegister().isRegistered("es.value.min")) {
    mMinValue = castHandleT<Vector>(ioSystem.getRegister()["es.value.min"]);
  } else {
    mMinValue = new Vector(1,DBL_MIN);
    std::ostringstream lOSS;
    lOSS << "ES vector minimum values. This can be a scalar, which limit the values for all ES, ";
    lOSS << "or a vector which limit the values individually. If the minimum value vector size is ";
    lOSS << "smaller than the ES vector size, the limit used for the last values of the ES vector ";
    lOSS << "is equal to the last value of the minimum value vector.";
    Register::Description lDescription(
      "Minimum ES vector values",
      "Vector",
      dbl2str(DBL_MIN),
      lOSS.str()
    );
    ioSystem.getRegister().addEntry("es.value.min", mMinValue, lDescription);
  }
}


/*!
 *  \brief Mutate an ES individual.
 *  \param ioIndividual ES individual to mutate.
 *  \param ioContext Context of the evolution.
 *  \return True as individual are always mutated.
 */
bool GA::MutationESVecOp::mutate(Beagle::Individual& ioIndividual, Context& ioContext)
{
  Beagle_ValidateParameterM(mMinStrategy->getWrappedValue()>=0.0,mMinStrategyName,"<0");

  Beagle_LogVerboseM(
    ioContext.getSystem().getLogger(),
    "mutation", "Beagle::GA::MutationESVecOp",
    std::string("Applying evolution strategy mutation to an individual")
  );

  for(unsigned int i=0; i<ioIndividual.size(); i++) {
    GA::ESVector::Handle lVector = castHandleT<GA::ESVector>(ioIndividual[i]);
    Beagle_LogVerboseM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("Mutating the ")+uint2ordinal(i+1)+" ES vector"
    );
    Beagle_LogDebugM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("The ES vector before mutation: ")+lVector->serialize()
    );

    const double lT = 1.0 / std::sqrt(2.0 * std::sqrt(double(lVector->size())));
    const double lTPrime = 1.0 / std::sqrt(2.0 * lVector->size());
    const double lN = ioContext.getSystem().getRandomizer().rollGaussian(0.0, 1.0);
    const double lMinStrategy = mMinStrategy->getWrappedValue();
    for(unsigned int j=0; j<lVector->size(); j++) {
      const double lMaxVal = j<mMaxValue->size() ? (*mMaxValue)[j] : mMaxValue->back();
      const double lMinVal = j<mMinValue->size() ? (*mMinValue)[j] : mMinValue->back();
      const double lNi = ioContext.getSystem().getRandomizer().rollGaussian(0.0, 1.0);
        
      (*lVector)[j].mStrategy *= std::exp((lTPrime * lN) + (lT * lNi));
      if((*lVector)[j].mStrategy < lMinStrategy) (*lVector)[j].mStrategy = lMinStrategy;
      (*lVector)[j].mValue += (*lVector)[j].mStrategy * lNi;
      if((*lVector)[j].mValue > lMaxVal) (*lVector)[j].mValue = lMaxVal;
      if((*lVector)[j].mValue < lMinVal) (*lVector)[j].mValue = lMinVal;
      
      Beagle_LogDebugM(
        ioContext.getSystem().getLogger(),
        "mutation", "Beagle::GA::MutationESVecOp",
        std::string("ES mutating by adding ")+dbl2str((*lVector)[j].mStrategy * lNi)+
        std::string(" to the value and multiplying the strategy by ")+
        dbl2str(std::exp((lTPrime * lN) + (lT * lNi)))+
        std::string(" to mutate the pair ")+uint2str(j)+
        std::string(" of the ES vector")
      );
    }
    
    Beagle_LogDebugM(
      ioContext.getSystem().getLogger(),
      "mutation", "Beagle::GA::MutationESVecOp",
      std::string("ES vector after mutation: ")+lVector->serialize()
    );
  }
  return true;
}
